####################################################################
#                                                                  #
#   CODE: Analyses                                                 #
#                                                                  #
#   Thia et al. (2018) Larval traits show temporally consistent    #
#   constraints, but are decoupled from post-settlement juvenile   # 
#   growth, in an intertidal fish. Journal of Animal Ecology.      #
#                                                                  #
####################################################################

# =========================================
# Setup the environment
# =========================================

# BEFORE RUNNING THIS CODE!

# You will require the following packages:
#   plspm
#   car
#   lubridate
#   data.table

# You will need to make sure all of the following files are in the current
# working directory:
#   Thia_2018_JAnimEcol_TraitCov_CODE_Functions_for_Analyses.R
#   Thia_2018_JAnimEcol_TraitCov_DATA_fish_info.csv
#   Thia_2018_JAnimEcol_TraitCov_DATA_pld_growth.csv
#   Thia_2018_JAnimEcol_TraitCov_DATA_post_set_growth.csv
#   Thia_2018_JAnimEcol_TraitCov_DATA_traits.csv


# Load the required libraries
libs <- c('plspm', 'car', 'lubridate', 'data.table')
for(i in libs){ require(i, character.only=TRUE); rm(i)}

# Get functions
source('Thia_2018_JAnimEcol_TraitCov_CODE_Functions_for_Analyses.R')


# =========================================
# Pull in the data
# =========================================
# Import otolith traits
oto.traits.raw <- read.table(file='Thia_2018_JAnimEcol_TraitCov_DATA_traits.csv', header=TRUE, row.names=NULL, sep=',')

# Import metadata: collection data and size
oto.id.dat <- read.table(file='Thia_2018_JAnimEcol_TraitCov_DATA_fish_info.csv', header=TRUE, row.names=NULL, sep=',')

# Import PLD growth data
oto.pld.grow.dat <- read.table(file='Thia_2018_JAnimEcol_TraitCov_DATA_pld_growth.csv', header=TRUE, row.names=NULL, sep=',')

# Import post-settlement growth data
oto.set.grow.dat <- read.table(file='Thia_2018_JAnimEcol_TraitCov_DATA_post_set_growth.csv', header=TRUE, row.names=NULL, sep=',')

# Turn pre-settlement growth data into list
grow.set <- FUN_uneven_df_2_list(oto.set.grow.dat, x.pre=TRUE)

# Remove any specimens that have no data. Cleaned variable (.cl)
grow.set.cl <- FUN_list_rm_less_than(grow.set, 4)

# Add in the 4th day after the settlement day.
# First create an empty column of NAs in oto.traits.raw
# Then create a dataframe of IDs and size at the 4th day.
# Match the IDs in oto.traits.raw with that in set.day4.size.df
oto.traits.raw$set.day4.size <- rep(NA, nrow(oto.traits.raw))
set.day4.size.df <- data.frame(id=as.integer(names(grow.set.cl))
                          , set.day4.size=do.call('rbind', lapply(grow.set.cl, function(X){ return(X[4])}))
                          )

for(i in oto.traits.raw$id){
  oto.traits.raw[match(i, oto.traits.raw$id), 'set.day4.size'] <- set.day4.size.df[match(i, set.day4.size.df$id), 'set.day4.size']
}

# Remove any rows with NAs from the final dataset, make it clean (.cl!), then sort by ID
oto.traits.cl <- na.omit(oto.traits.raw)
oto.traits.cl <- oto.traits.cl[order(oto.traits.cl$id), ]

# Now add collection information (year, month, day, size)
oto.traits.cl <- FUN_match_dataframes(oto.traits.cl, oto.id.dat, c(2:5), 1, 1)

# Generate the growth data
# ------------------------+
oto.traits.cl$grow.pelagic <- oto.traits.cl$set.size - oto.traits.cl$hatch.size
oto.traits.cl$grow.set <- oto.traits.cl$set.day4.size - oto.traits.cl$set.size

# Create year colour vector
# -------------------------+
year.cols <- factor()
for (i in oto.traits.cl$year){
  if (i == "2014"){ year.cols <- c(year.cols, "#F9AD43")}
  if (i == "2015"){ year.cols <- c(year.cols, "#34CAD1")}
  if (i == "2016"){ year.cols <- c(year.cols, "#9C3DAA")}
}
oto.traits.cl$year.cols <- year.cols; rm(year.cols)

# Add hatch date information into the dataset
# -------------------------------------------+
# Create a vector of dates when the fish were collected
date.vec <- character()
for (i in seq(1, nrow(oto.traits.cl))){
  temp <- paste(oto.traits.cl$year[i],oto.traits.cl$month[i],oto.traits.cl$day[i],sep="-")
  date.vec <- c(date.vec,temp)
}
date.vec <- as.Date(date.vec)

# The hatch date is the date collected minus the estimated age of the fish
hatch.date <- date.vec - oto.traits.cl$age.post.hatch
oto.traits.cl$hatch.year <- year(hatch.date)
oto.traits.cl$hatch.month <- unlist(lapply(as.list(month(hatch.date)), function(X){
  ifelse(X==12, return('Dec'), ifelse(X==1, return('Jan'), ifelse(X==2, return('Feb'))))
}))
oto.traits.cl$hatch.day <- mday(hatch.date)

# Create a hatch date guide to match against real dataset
day.ref <-data.frame(
  month.day=paste0(c(rep('Dec',31), rep('Jan',31), rep('Feb',28)),1:31)
  , day.index=1:90
)

# Match dataset against hatch date guide
vec.day <- integer()
for (i in 1:nrow(oto.traits.cl)){
  X <- oto.traits.cl[i,]
  Y <- paste0(X$hatch.month,X$hatch.day)
  Z <- match(Y, day.ref$month.day)
  vec.day <- c(vec.day,day.ref$day.index[Z])
}

# Combine the numerical month and day values into a measure of "season"
# oto.traits.cl$hatch.date <- day.vec + month.vec
oto.traits.cl$hatch.date <- vec.day

# Add breeding 'season' to the dataset
# ------------------------------------+
season <- character()
for (i in oto.traits.cl$year){
  if (i=='2014'){ season <- c(season, '2013/14')}
  if (i=='2015'){ season <- c(season, '2014/15')}
  if (i=='2016'){ season <- c(season, '2015/16')}
}
oto.traits.cl$season <- season; rm(season)


# =========================================
# Write the cleaned dataset to file
# =========================================
# write.csv(oto.traits.cl, file='Thia_2018_JAnimEcol_TraitCov_DATA_traits_cleaned.csv', row.names=FALSE)

# =========================================
# Temporal trait variation
# =========================================
# Calculate the coefficient of variation in each trait, pooled over years
apply(subset(oto.traits.cl
              , select=c('hatch.size', 'pld', 'grow.pelagic'
                         , 'set.size', 'grow.set', 'set.day4.size')
             )
      , 2
      , function(X){return(sd(X)/mean(X))}
      )

# ANCOVA - with interactions
file.create('Thia_2018_JAnimEcol_TraitCov_RESULTS_Model_HatchdateBySeason_ANCOVA_WithInteraction.txt')
for (i in c('hatch.size', 'pld', 'grow.pelagic', 'grow.set', 'set.size', 'set.day4.size')){
  sink(file='Thia_2018_JAnimEcol_TraitCov_RESULTS_Model_HatchdateBySeason_ANCOVA_WithInteraction.txt', append=T)
  cat("=================",i,"=================",sep="\n")
  trait <- oto.traits.cl[[i]]; season <- as.factor(oto.traits.cl$season); hatch.date <- oto.traits.cl$hatch.date
  mod <- lm(trait ~ season * hatch.date + season:hatch.date)
  print(summary(mod))
  cat("-----------------",sep="\n")
  print(Anova(mod, type=2))
  cat(sep="\n"); cat(sep="\n")
  sink()
}

# ANCOVA - without interactions
file.create('Thia_2018_JAnimEcol_TraitCov_RESULTS_Model_HatchdateBySeason_ANCOVA_NoInteraction.txt')
for (i in c('hatch.size', 'pld', 'grow.pelagic', 'grow.set', 'set.size', 'set.day4.size')){
  sink(file='Thia_2018_JAnimEcol_TraitCov_RESULTS_Model_HatchdateBySeason_ANCOVA_NoInteraction.txt', append=T)
  cat("=================",i,"=================",sep="\n")
  trait <- oto.traits.cl[[i]]; season <- as.factor(oto.traits.cl$season); hatch.date <- oto.traits.cl$hatch.date
  mod <- lm(trait ~ season + hatch.date)
  print(summary(mod))
  cat("-----------------",sep="\n")
  print(Anova(mod, type=2))
  cat(sep="\n"); cat(sep="\n")
  sink()
}


# =========================================
# Correlations between size and growth
# =========================================
cor.mat <- cor(oto.traits.cl[, c('hatch.size', 'grow.pelagic', 'set.size', 'grow.set', 'set.day4.size')])
write.csv(cor.mat, file='Thia_2018_JAnimEcol_TraitCov_RESULTS_GrowthAndSize_Correlations.csv', row.names=T)

pairs(oto.traits.cl[, c('hatch.size', 'grow.pelagic', 'set.size', 'grow.set', 'set.day4.size')]
      , pch=21, cex=1.4, bg=as.character(oto.traits.cl$year.cols), upper.panel=NULL)

# =========================================
# Path models
# =========================================

# Setup model components
# -------------------------+
# What is the data?
path.data <- data.frame(subset(oto.traits.cl
                                  , select=c("hatch.size"
                                             , "pld"
                                             , "grow.pelagic"
                                             , "grow.set"
                                             , 'set.size'
                                             , 'set.day4.size'
                                             , "hatch.date"
                                             , "season"
                                             , "year")

  )
)

# Setup different matrices:
HatchSize <- c(0,0,0,0)
PLD <-c(1,0,0,0)
PelagicGrowth <- c(1,1,0,0)
SetGrowth <- c(1,1,1,0)

# Create the variable blocks
pathmat <- rbind(HatchSize, PLD, PelagicGrowth, SetGrowth)

# Setup the indexes for each block
pathblock <- list(1,2,3,4)

# Pooled samples
# --------------+
# Full model: all traits separate
pathmod.pool <- plspm(path.data, pathmat, pathblock, c("A","A","A","A"), scaled=T, boot.val=T, br=1000)
summary(pathmod.pool)


# Individual seasons
# ------------------+
# Full models
pathmod.14 <- plspm(subset(path.data, season == "2013/14"), pathmat, pathblock, c("A","A","A","A"), scaled=T, boot.val=T, br=1000)
pathmod.15 <- plspm(subset(path.data, season == "2014/15"), pathmat, pathblock, c("A","A","A","A"), scaled=T, boot.val=T, br=1000)
pathmod.16 <- plspm(subset(path.data, season == "2015/16"), pathmat, pathblock, c("A","A","A","A"), scaled=T, boot.val=T, br=1000)

sum.14 <- summary(pathmod.14); sum.14
sum.15 <- summary(pathmod.15); sum.15
sum.16 <- summary(pathmod.16); sum.16

# Combine the data together
# -------------------------+
label.paths <- c("HS-LD", "HS-PG", "HS-SG", "LD-PG", "LD-SG", "PG-SG")
name.paths <- c("Hatch size on\nLarval duration", "Hatch size on\nPelagic growth" , "Hatch size on\nPost-settlement growth"
                , "Larval duration on\nPelagic growth", "Larval duration on\nPost-settlement growth"
                ,"Pelagic growth on\nPost-settlement growth")

# Export the direct effects
effs.dir <- rbind(pathmod.14$boot$path, pathmod.15$boot$path, pathmod.16$boot$path, pathmod.pool$boot$path)
effs.dir$year <- c(rep("2013/14", 6), rep("2014/15", 6), rep("2015/16", 6), rep("Pooled", 6))
effs.dir$paths <- rep(label.paths, 4)
effs.dir$names <- rep(name.paths,4)
rownames(effs.dir) <- NULL
write.csv(effs.dir, 'Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_DirectEffects.csv', row.names=F)

# Export the total effects
effs.tot <- rbind(pathmod.14$boot$total.efs, pathmod.15$boot$total.efs, pathmod.16$boot$total.efs, pathmod.pool$boot$total.efs)
effs.tot$year <- c(rep("2013/14", 6), rep("2014/15", 6), rep("2015/16", 6), rep("Pooled", 6))
effs.tot$paths <- rep(label.paths, 4)
effs.tot$names <- rep(name.paths,4)
rownames(effs.tot) <- NULL
write.csv(effs.tot, "Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_TotalEffects.csv", row.names=F)

# Export the R2
trait.r2 <- data.frame(R2=c(sum.14$inner_summary$R2, sum.15$inner_summary$R2, sum.16$inner_summary$R2, summary(pathmod.pool)$inner_summary$R2))
trait.r2$year <- c(rep("2013/14", 4), rep("2014/15", 4), rep("2015/16", 4), rep("Pooled", 4))
trait.r2$traits <- rep(c("Hatch size", "Larval duration", "Pelagic growth", "Post-settlement growth"))
rownames(trait.r2) <- NULL
write.csv(trait.r2, "Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_R2.csv",row.names=F)

# Test groups, pairwise
# ---------------------+
# tests.dir = direct effects between years
# tests.tot = total effects between years
year.comps <- combn(c('2013/14', '2014/15', '2015/16'), 2)
tests.dir <- data.frame(matrix(NA, 0, 4))
colnames(tests.dir) <- c('comparison', 'path', 't.test', 'pval')
tests.tot <- data.frame(matrix(NA, 0, 4))
colnames(tests.tot) <- c('comparison', 'path', 't.test', 'pval')

# Conduct Welch's t-test for path coefficients in pairs of years
for(i in unique(effs.dir$paths)){
  for(j in 1:ncol(year.comps)){
    tests.dir <- rbind(
      tests.dir
      , FUN_tTest_paths_years_diff_var(
        dat.paths=effs.dir
        , sub.years=year.comps[,j]
        , sub.path=i
        , bootstraps=c(1000, 1000)
      )
    )
  }
}

setorder(x=tests.dir, cols=comparison)
write.csv(tests.dir, 'Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_SeasonTests_DirectEffects.csv', row.names=F)

# Conduct Welch's t-test for total effects between years
for(i in unique(effs.tot$paths)){
  for(j in 1:ncol(year.comps)){
    tests.tot <- rbind(
      tests.tot
      , FUN_tTest_paths_years_diff_var(
        dat.paths=effs.tot
        , dat.sizes=sample.sizes
        , sub.years=year.comps[,j]
        , sub.path=i
        , bootstraps=c(1000, 1000)
      )
    )
  }
}

setorder(x=tests.tot, cols=comparison)
write.csv(tests.tot, 'Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_SeasonTests_TotalEffects.csv', row.names=F)
