####################################################################
#                                                                  #
#   CODE: Figures                                                  #
#                                                                  #
#   Thia et al. (2018) Larval traits show temporally consistent    #
#   constraints, but are decoupled from post-settlement juvenile   # 
#   growth, in an intertidal fish. Journal of Animal Ecology.      #
#                                                                  #
####################################################################

# =========================================
# Set up the environment
# =========================================

# BEFORE RUNNING THIS CODE!

# You will require the following packages:
#   gridExtra
#   ggplot2

# You will need to make sure all of the following files are in the current
# working directory:
#   Thia_2018_JAnimEcol_TraitCov_CODE_Functions_for_Figures.R
#   Thia_2018_JAnimEcol_TraitCov_DATA_Otolith_Traits_Cleaned.csv
#   Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_DirectEffects.csv
#   Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_TotalEffects.csv

# The RESULTS files indicated above can be generated from running the script:
#   Thia_2018_JAnimEcol_TraitCov_CODE_Analyses.R

library(gridExtra); library(ggplot2)

# Get functions
source('Thia_2018_JAnimEcol_TraitCov_CODE_Functions_for_Figures.R')

# Get some data (or run Thia_2018_JAnimEcol_TraitCov_CODE_Analyses.R)
oto.traits.cl <- read.csv(file='Thia_2018_JAnimEcol_TraitCov_DATA_traits_cleaned.csv')
effs.dir <- read.csv('Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_DirectEffects.csv')
effs.tot <- read.csv('Thia_2018_JAnimEcol_TraitCov_RESULTS_PathMods_Coefficients_TotalEffects.csv')

# Plotting variables
name.paths <- c("Hatch size on\nLarval duration", "Hatch size on\nPelagic growth" , "Hatch size on\nPost-settlement growth"
                , "Larval duration on\nPelagic growth", "Larval duration on\nPost-settlement growth"
                ,"Pelagic growth on\nPost-settlement growth")

label.paths <- c("HS-LD", "HS-PG", "HS-SG", "LD-PG", "LD-SG", "PG-SG")


# =========================================
# My colour palette
# =========================================
# Temporal yellows
# cols.years = c("#ffe6a5ff", "#ffd055ff", "#ffba05ff")
cols.years <- c("#F9AD43","#34CAD1", "#9C3DAA")
cols.slopes <- c("#ffce8e","#79e6ea", "#be74c9", "black")
names(cols.years) <- c("2013/14", "2014/15", "2015/16")


# =========================================
# Demonstrate the relationship between 
# age, standard length, and max otolith 
# size
# =========================================
pairs(subset(oto.traits.cl, select=c("age.post.hatch","size","max.oto.size")), pch=21, bg=as.character(oto.traits.cl$year.cols), cex=2
  , lower.panel=panel.regression, upper.panel=NULL)


# =========================================
# Plot the distribution of hatch dates 
# and days post-settlement
# =========================================
FUN_fig_box_col_x(oto.traits.cl, in.x="season", in.y="hatch.date", in.xlab="Breeding season", in.ylab="Hatch date"
  , in.main="A", in.cols=cols.years, make.legend=F) + coord_flip() + theme(plot.title = element_text(face='bold'))

FUN_fig_hist(in.data=oto.traits.cl, in.x="age.post.set", in.xlab = "Days post-settlement", in.ylab = "Frequency", in.main="B"
  , in.bins = 25, in.xlim = NULL, in.ylim = NULL, in.cols = "#bcbcbc") + theme(plot.title = element_text(face='bold'))

FUN_fig_box_col_x(in.data=oto.traits.cl, in.x="season", in.y="age.post.set", in.xlab="Breeding season", in.ylab="Days post-settlement"
  ,in.main="C", in.cols=cols.years, make.legend=F) + coord_flip() + theme(plot.title = element_text(face='bold'))


# =========================================
# Plot correlation between post-set size 
# and growth, and show change in size
# at different life-stages
# =========================================
# Create a data frame that holds the size of individuals at difference stages.
# 1 = hatch, 2 = settlement, 3 = day 4 post-settlement.
stage.size <- rbind(data.frame(stage=1, id=oto.traits.cl$id, size=oto.traits.cl$hatch.size)
                    , data.frame(stage=2, id=oto.traits.cl$id, size=oto.traits.cl$set.size)
                    , data.frame(stage=3, id=oto.traits.cl$id, size=oto.traits.cl$set.day4.size)
)

# Make grid of 2 figs: 
#   A) correlation between post-set growth and size;
#   B) size at different life-stages
grid.arrange(
    (ggplot(oto.traits.cl, aes(x=grow.set, y=set.day4.size))
     + my.theme
     + theme(plot.title = element_text(face='bold'))
     + theme(plot.margin = unit(c(5.5, 5.5, 10, 5.5), 'pt'))
     + geom_point(size=2, shape=16, colour='black')
     + xlab('Post-settlement growth (um)')
     + ylab('Post-settlement size (um)')
     + ggtitle('A')
     + annotate('text',  x=27.5, y=68, label=paste0('R2 = ', 0.272))
    )
  , (ggplot(stage.size, aes(x=as.factor(stage), y=size))
     + my.theme
     + theme(plot.title = element_text(face='bold'))
     + theme(plot.margin = unit(c(10, 5.5, 5.5, 5.5), 'pt'))
     + geom_line(data=stage.size, aes(x=stage, y=size, group=id))
     + geom_point(size=2, shape=16, colour='black')
     + xlab('Life-stage')
     + ylab('Size (um)')
     + scale_x_discrete(labels=c('Hatch', 'Settlement', 'Post-settlement'))
     + ggtitle('B')
    )
  , ncol=1
)


# =========================================
# Path models: plot direct effects
# =========================================
for (i in label.paths){
  print(i)
  path.num <- which(label.paths == i)
  gg <- FUN_plot_dots_w_error(subset(effs.dir, paths==i), "year", "Mean.Boot"
                          , c(cols.years,'black'), c("perc.025","perc.975")
                          , in.title = i, in.space=0, in.dotsize=3.5)
  plot(gg + theme(axis.text.x=element_blank(), axis.ticks.x=element_blank()) 
          + scale_y_continuous(limits=c(-1,1)
                               , breaks=c(-1, -0.5, 0, 0.5, 1)
                               , labels=c(-1, '', 0, '', 1)
                               )
       )
}


# =========================================
# Path models: plot total effects
# =========================================
for (i in label.paths){
  print(i)
  path.num <- which(label.paths == i)
  gg <- FUN_plot_dots_w_error(subset(effs.tot, paths==i), "year", "Mean.Boot"
                              , c(cols.years,'black'), c("perc.025","perc.975")
                              , in.title = i, in.space=0, in.dotsize=3.5)
  plot(gg + theme(axis.text.x=element_blank(), axis.ticks.x=element_blank()) 
          + scale_y_continuous(limits=c(-1,1)
                               , breaks=c(-1, -0.5, 0, 0.5, 1)
                               , labels=c(-1, '', 0, '', 1)
                               , position='right'
                               )
       )
}
