####################################################################
#                                                                  #
#   CODE: Functions for Analyses                                   #
#                                                                  #
#   Thia et al. (2018) Larval traits show temporally consistent    #
#   constraints, but are decoupled from post-settlement juvenile   # 
#   growth, in an intertidal fish. Journal of Animal Ecology.      #
#                                                                  #
####################################################################

FUN_list_rm_less_than <- function(in.list, N){
# This function is used to remove any items from a list that have a length
# less than N, e.g. 
# 
#   > my.list
#     $'var1'
#       [1] 1   2
#     $'var2'
#       [1] 1   2   3   4   
# 
# As per above, 'var1' would be removed if N = 2, but 'var2' would be kept.
# 
# INPUTS
#   in.list   (list) An input list
#   N         (int) A single value; the desired vector minimum length.
# 
# OUTPUTS
#   A list: specifically, in.list but with all items with length of '0'
#   removed.
# 
# USAGE
#   new.list <- FUN_list_rm_empty(original.list)
  
  out.list <- list()
  
  indices <- names(in.list)
  
  for (i in indices){
    
    if (length(in.list[[i]]) > N){
      out.list[[i]] <- in.list[[i]]
    }
  }
  
  return(out.list)
}

FUN_match_dataframes <- function(df.sub, df.que, cols.que, id.sub, id.que){
  # This function will takes a SUBJECT dataframe to compare against a QUERY dataframe.
  # It is assumed you want to match data from the QUERY and add it (as columns) to 
  # the subject. 
  # 
  # Matching is done using a shared 'ID' column between the dataframes. It is 
  # expected that the QUERY contains some, if not all, the IDs present in the
  # SUBJECT.
  # 
  # The function essentially reorganises the QUERY dataframe such that the rows are in
  # the same order as the SUBJECT (based on the orders of IDs in the SUBJECT). When
  # a SUBJECT ID is absent from the QUERY, a row of NAs is inserted.
  # 
  # The user must specify the columns (same order as the QUERY) they want to 
  # append to the SUBJECT dataframe, as well as the columns that contain the IDs
  # in both the SUBJECT and QUERY.
  # 
  # INPUTS
  #   df.sub    (dataframe) The SUBJECT dataframe
  #   df.que    (dataframe) The QUERY dataframe
  #   cols.que  (numeric) A numeric vector indicating the column numbers in the QUERY
  #               that the user wants to transfer to the SUBJECT.
  #   id.sub    (int) A value indicating the column number for SUBJECT IDs
  #   id.que    (int) A value indicating the column number for QUERY IDs
  # 
  # OUTPUTS
  #   Returns a dataframe that is a copy of df.sub but with the specified columns form
  #   df.que appended on the end.
  # 
  # USAGE
  #   new.df <- FUN_match_dataframes(df1, df2, c(2, 3), 1, 1)
  
  # Setup a temporary dataframe; this will be used to store matched QUERY rows
  df.temp <- as.data.frame(matrix(data = NA, nrow = 0, ncol = ncol(df.que)))
  colnames(df.temp) <- colnames(df.que)
  
  # Make a single row dataframe of NAs
  df.NA <- rbind(rep(NA, times = ncol(df.que)))
  colnames(df.NA) <- colnames(df.que)
  
  # Search for each of the SUBJECT IDs in the QUERY
  for (i in df.sub[, id.sub]){
    
    # What row was the ID found in the QUERY?
    j <- which(df.que[, id.que] == i)
    
    if (length(j) == 0){ df.temp <- rbind(df.temp, df.NA) }
    else if (length(j) == 1){ df.temp <- rbind(df.temp, df.que[j, ]) }
    else if (length(j) > 1){ stop("There are multiple instances of the same ID!") }
    
  }
  
  # The output dataframe will be SUBJECT + extras
  # Append the specified QUERY columns stored in df.temp to the SUBJECT 
  df.out <- cbind(df.sub, df.temp[, cols.que])
  
  return(df.out)
}


FUN_uneven_df_2_list <- function(in.df, x.pre = FALSE){
# This function is used to take a dataframe, where column lengths are
# uneven, and convert it into a list. 
# 
# The function is specifically meant to deal with a data structure where
# individual specimens are in the columns (i.e. the header name is 
# the specimen ID). All columns have measurements for the same variable, 
# but perhaps there are unequal numbers of observations for each specimen.
# 
# The function removes any "NA" that may be in the column and returns
# only the numerical values.
# 
# R will import numerical header names (e.g. "123") with an "X" prefix 
# (e.g. "X123"). The function is able to remove this.
# 
# INPUTS
#   in.df     (dataframe) The input dataframe; specimens in columns, 
#               headers as specimen IDs.
#   x.pre     (boolean) Whether or not the function should remove a 
#               "X" prefix from the specimen IDs; default is FALSE.
# 
# OUTPUTS
#   A list where each items is the values of each column, sans the "NA".
# 
# USAGE
#   my.list <- FUN_uneven_df_2_list(my.data, x.pre = TRUE)
  
  library(stringr)
  
  # Get the header names; i.e. the specimen IDs
  head.names <- colnames(in.df)
  
  # Make empty list
  out.list <- list()
  
  # Iterate through in.df with IDs
  for (i in head.names){
    
    # Extract column to vector, then remove NA
    x <- in.df[, i]
    x <- x[!is.na(x)]
    
    # Do you need to remove an 'X' prefix?
    if (x.pre == TRUE){
      N <- str_split_fixed(i, "X", n = 2)
      N <- N[, 2]
    }
    
    # If you don't need to remove an 'X' prefix
    else if (x.pre == FALSE){
      N <- i
    }
    
    # Write the vector to list, using ID as a list index
    out.list[[N]] <- x
  }
  
  return(out.list)
}

FUN_tTest_paths_years_diff_var <- function(dat.paths, dat.sizes, sub.years, sub.path, bootstraps){
  # Subsets the data to consider a pair of years and a single path.
  # Use this when variances are different between years.
  # Conducts a Welch t-test: https://en.wikipedia.org/wiki/Welch%27s_t-test
  # Link for refs and df explanation: https://stats.stackexchange.com/questions/48636/are-the-degrees-of-freedom-for-welchs-test-always-less-than-the-df-of-the-poole
  # See here for manual pval calculation: http://www.cyclismo.org/tutorial/R/pValues.html
  #
  # INPUTS:
  #   dat.paths .... The input data: contains bootstrapped path coefficients and
  #                   associated statistics. Requires the columns:
  #                   $year = the sampling year
  #                   $Std.Error = the standard error of the coefficient mean
  #                   $Mean.Boot = the bootstrap mean of the coefficients
  #                   $path = the pathways
  #   sub.years .... The years to subset by
  #   sub.path ..... The path to subset by
  #	  bootstraps ... The number of bootstrap replicates for each year
  #
  # OUTPUTS:
  #   A dataframe:
  #     $comparison = the years compared.
  #     $path = the pathway analysed
  #     $t.stat = the t-statistic for comparison between years
  #     $pval = the two-taiiled p-value for the associated t-statistic
  
  # Sample sizes
  n1 <- bootstraps[1]
  n2 <- bootstraps[2]
  
  # Standard errors
  SE1 <- subset(dat.paths, year==sub.years[1] & paths==sub.path)$Std.Error
  SE2 <- subset(dat.paths, year==sub.years[2] & paths==sub.path)$Std.Error
  
  # Calculate the standard deviations
  SD1 <- sqrt(n1) * SE1
  SD2 <- sqrt(n2) * SE2
  
  # Path coefficients
  b1 <- subset(dat.paths, year==sub.years[1] & paths==sub.path)$Mean.Boot
  b2 <- subset(dat.paths, year==sub.years[2] & paths==sub.path)$Mean.Boot
  
  # Calculate the degrees of freedom
  df1 <- n1 - 1
  df2 <- n2 - 1
  
  df.numer <- ((SD1^2/n1) + (SD2^2/n2))^2
  df.denom <- (SD1^4/(n1^2 * df1)) + (SD2^4/(n2^2 * df2)) 
  
  DF <- round(df.numer/df.denom)
  
  # Calculate the t-statistic and get a p-value.
  t.stat <- (b1-b2)/(sqrt((SD1^2/n1) + (SD2^2/n2)))
  pval <- 2 * pt(-abs(t.stat), df=DF)
  
  return(
    data.frame(comparison=paste(sub.years[1], sub.years[2], sep=' vs. ')
               , path=sub.path
               , t.stat=t.stat
               , df = DF
               , pval=pval
               , row.names = NULL
    )
  )
}