####################################################################
#                                                                  #
#   CODE: Functions for Figures                                    #
#                                                                  #
#   Thia et al. (2018) Larval traits show temporally consistent    #
#   constraints, but are decoupled from post-settlement juvenile   # 
#   growth, in an intertidal fish. Journal of Animal Ecology.      #
#                                                                  #
####################################################################


# This here maps the Windows font "Gill Sans MT" into R, which can now be called as a font family.
my.theme <- theme_classic() + theme(
  text = element_text(colour = "black", size = 16)
  ,plot.title = element_text(margin=margin(t = 0, r = 0, b = 8, l = 0, unit = "pt"), hjust = 0.5, size=16)
  ,axis.text = element_text(colour = "black")
  ,axis.title.x = element_text(margin = margin(t = 10, r = 0, b = 0, l = 0, unit = "pt"))
  ,axis.title.y = element_text(margin = margin(t = 0, r = 10, b = 0, l = 0, unit = "pt"))
  # ,axis.text.y = element_text(angle=90, vjust=1)
  ,axis.ticks = element_line(size = 0.5, colour = "black")
  ,axis.ticks.length = unit(5, "pt")
  ,legend.title = element_blank()
  ,axis.line.x = element_blank()
  ,axis.line.y = element_blank()
  ,panel.background = element_rect(fill=NULL)
  ,panel.border = element_rect(fill=NA, size=0.5)
)

# This code here is a sneaky function to conduct linear regressions in the pairs() function
panel.regression <- function (x, y, col = par("col"), bg = NA, pch = par("pch"), cex = 1, col.regres = "black", lwd=2, ...)
{
  points(x, y, pch = pch, col = col, bg = bg, cex = cex)
  ok <- is.finite(x) & is.finite(y)
  if (any(ok))
    abline(stats::lm(y[ok] ~ x[ok]), col = col.regres, ...)
}

# Other functions
FUN_fig_box_col_x <- function(
  in.data, in.x, in.y, in.xlab = "X var", in.ylab = "Y var", in.main = NULL
  , in.xlim = NULL, in.ylim = NULL, in.title=NULL, in.cols = "#bcbcbc", make.legend = TRUE)
{
  temp.df <- data.frame(x_var = in.data[[in.x]], y_var = in.data[[in.y]])
  out.fig <- ggplot(data = temp.df, aes(x = as.factor(x_var), y = y_var, fill = as.factor(x_var)))
  out.fig <- (out.fig + my.theme
              + stat_boxplot(geom='errorbar', linetype = 1, size = 0.5, width = 0.25)
              + geom_boxplot(size = 0.5, , colour = "black")
              + xlab(in.xlab) + ylab(in.ylab) + ggtitle(in.title)
  )
  if (length(in.main) > 0){
    out.fig <- out.fig + ggtitle(in.main)
  }
  if (make.legend == FALSE){
    out.fig <- out.fig + theme(legend.position = "none")
  }
  if (length(in.xlim) == 2){
    out.fig <- out.fig + xlim(in.xlim[1], in.xlim[2])
  }
  if (length(in.ylim) == 2){
    out.fig <- out.fig + ylim(in.ylim[1], in.ylim[2])
  }
  if (length(in.cols) > 0){
    out.fig <- out.fig + scale_fill_manual(values = in.cols)
  }
  return(out.fig)
}

FUN_fig_hist <-  function(
  in.data, in.x, in.xlab = "X var", in.ylab = "Frequency", in.main = NULL, in.bins = 25
  , in.xlim = NULL, in.ylim = NULL, in.cols = "#bcbcbc")
{
  temp.df <- data.frame(x_var = in.data[[in.x]])
  out.fig <- ggplot(data = temp.df, aes(x = x_var))
  out.fig <- (out.fig + my.theme
              + geom_histogram(size = 0.5, colour = "black", fill = in.cols, bins = in.bins)
              + xlab(in.xlab) + ylab(in.ylab)
  )
  if (length(in.main) > 0){
    out.fig <- out.fig + ggtitle(in.main)
  }
  if (length(in.xlim) == 2){
    out.fig <- out.fig + coord_cartesian(xlim = c(in.xlim[1], in.xlim[2]))
  }
  if (length(in.ylim) == 2){
    out.fig <- out.fig + coord_cartesian(ylim = c(in.ylim[1], in.ylim[2]))
  }
  out.fig <- out.fig + scale_x_continuous(breaks = pretty(temp.df$x_var, n = 6))
  return(out.fig)
}


FUN_plot_dots_w_error <- function(in.data, in.x, in.y, in.cols, in.error, in.dotsize = 8
                                  , in.xlab = NULL, in.ylab = NULL, in.title = NULL, in.space = 0){

  # Function plots a point for every value in.y value realted to in.x (assumed in.x represents groups).
  # Error bars are also plotted.
  #
  # in.data        (dataframe) Input data.
  # in.x           (character) Column name for the x-values, assumed to be groups, each has a single y-value.
  # in.y           (character) Column name for the y-values, one value for each unique value in in.x.
  # in.error       (character) A vector of two values: 1 = column name of lower error bar values,
  #                              2 = column name of higher error bar values.
  # in.dotsize     (numeric)   Size of dots.a
  # in.xlab        (character) Name for x-axis.
  # in.ylab        (character) Name for y-axis.
  # in.title       (character) Name for title.
  # in.space       (numeric)   Used to create space above/below points. This value is added to the lowest
  #                              and highest error values to set the y-axis limits.

  out.fig <- ggplot(data = in.data, aes_string(x = in.x, y = in.y))
  out.fig <- (out.fig + my.theme
              # + theme(axis.text.x = element_text(angle = 90, hjust = 1))
              + geom_hline(yintercept=0, size=0.5, linetype = 2, colour="#969696")
              + geom_errorbar(aes(ymin=in.data[[in.error[1]]], ymax=in.data[[in.error[2]]]), linetype = 1, size = 0.5, width = 0)
              + geom_point(pch = 21, size = in.dotsize, fill = in.cols, stroke = 0.75)
              + xlab(in.xlab) + ylab(in.ylab) + ggtitle(in.title)
              + ylim(min(in.data[[in.error[1]]])-in.space, max(in.data[[in.error[2]]])+in.space)
              
  )

  return(out.fig)
}